/*!
 * (C) 2002 "Kaspersky Lab"
 *
 * \file EVP/EventsProcessor.h
 * \author  
 * \date 2002
 * \brief ,    ,   
 *          .
 *
 */

#ifndef KLEVP_EVENTSPROCESSOR_H
#define KLEVP_EVENTSPROCESSOR_H

#include <vector>
#include <string>

#include <std/base/klbase.h>
#include <kca/prci/componentid.h>
#include <srvp/evp/eventproperties.h>
#include <srvp/evp/eventseverities.h>

namespace KLEVP {

    //!    ,   EventsProcessor.
    struct EventInfo
    {
		std::wstring eventId; ///< GUID -  .
        KLPRCI::ComponentId publisher; ///<  ,  .
		std::wstring eventType;		   ///<   .     
		KLSTD::CAutoPtr<KLPAR::Params> body;  ///<  .
        time_t  eventTime;                    ///<   .
        long    eventTimeMsec;                  ///<   ()
        std::wstring subscription; ///<    .
		EventInfo():eventTime(0), eventTimeMsec(0){}
    };

    const wchar_t pcszEventInfoArray[] = L"EventInfoArray";

	const wchar_t pcszEventId[] = L"EventId";
	const wchar_t pcszEventType[] = L"EventType";
	const wchar_t pcszBody[] = L"Body";
	const wchar_t pcszEventTime[] = L"EventTime";
    const wchar_t pcszEventTimeMs[] = L"EventTimeMsec";
	const wchar_t pcszSubscription[] = L"Subscription";
	const wchar_t pcszComponentId_PRODUCT_NAME[] = L"COMPONENT_ID_PRODUCT_NAME";
	const wchar_t pcszComponentId_VERSION[] = L"COMPONENT_ID_VERSION";
	const wchar_t pcszComponentId_COMPONENT_NAME[] = L"COMPONENT_ID_COMPONENT_NAME";
	const wchar_t pcszComponentId_INSTANCE_ID[] = L"COMPONENT_ID_INSTANCE_ID";

    /*!
    * \brief      , 
    *         ( EventsManager)   .
    *
    *         Process    
    *    :
    *
    *   1)       . 
    *   2) ,       ,
    *         persistent      
    *       (SNMP, net send, run executable).
    *   3) ,        ,  
    *       persistent .
    *   4) ,      (  
    *        ), 
    *        persistent .     
    *      callback-,     .
    *   5)  ,  ,     
    *          .
    *   6)         EventsManager.
    *
    *    persistent- ( ,      ,
    *    )      
    *  .
    *
    *         
    *   .
    *
    *  1)   ,       
    *         .
    *  2)  SNMP  net send -      
    *        timeout,   timeout   
    *          .
    *  3)         
    *         ,   
    *          .
    *
    *             bulk insert:
    *
    *  1)      ,   .  
    *          ,   .
    *  2)         ,  
    *                .
    *                .
    *  3)         - ,   
    *        ,   .
    * 
    *  
    *          :
    *  
    *  1)      ,   .
    *          ,   .
    *  2) ,         .
    *            , 
    *      .       
    *     ,    ,    
    *              timeout.
    *  3)       , 
    *             .  
    *         , 
    *           ,    deadlock 
    *        .
    *  4)        commit .
    *       .
    *  5)       rollback ,
    *              
    *         .       
    *        .
    *
    */

    
    class EventsProcessor : public KLSTD::KLBase {
    public:

    /*!
      \brief        .

        \param package      [in]    .
        \param deliveryId   [in]   ,  
                                   .
        \param domain       [in]   ,   .
        \param hostName     [in]    ,   .
        \param agentId      [in]   , 
                                  .
       
    */
        virtual void Process ( 
            const std::vector<EventInfo> & package,
            const std::wstring & deliveryId,
            const std::wstring & domain,
            const std::wstring & hostName,
            const std::wstring & agentId ) = 0;
    };
    
    class EventsProcessorFactory : public KLSTD::KLBase
    {
    public:
        virtual void CreateInstance( EventsProcessor ** pEventsProcessor ) = 0;
    };
}

#endif // KLEVP_EVENTSPROCESSOR_H

