// TreeWnd.cpp : implementation file
//

#include "stdafx.h"
#include <tchar.h>
#include "WASCRes.rh"
#include <WASCMess.h>
#include <Stuff\SArray.h>
#include <StuffIO\Utils.h>
#include "TreeWnd.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define getKState(p) (::GetKeyState(p) & 0x8000)

/////////////////////////////////////////////////////////////////////////////
// CTreeWnd
// ---
CTreeWnd::CTreeWnd() :
  lastAddNode( NULL ),
  lastAddLevel( 0 ),
	notRedrawCount( 0 ),
	m_bDeleteCollapsedSubTree( false ),
	m_bPossibleRedraw( true ) {
}

// ---
CTreeWnd::~CTreeWnd()	{
}

// ---
void CTreeWnd::SetRedraw( BOOL bRedraw ) {
	if ( !bRedraw ) {
		if ( !notRedrawCount )
			CTreeCtrl::SetRedraw( FALSE );
		notRedrawCount++;
	}
	else {
		if ( notRedrawCount ) 
			notRedrawCount--;
		if ( !notRedrawCount && IsPossibleRedraw() ) 
			CTreeCtrl::SetRedraw( TRUE );
	}
}

//      
// ---
HTREEITEM CTreeWnd::RecourseEnumeration( HTREEITEM hNode, int &index ) {
  HTREEITEM result = hNode;
  HTREEITEM child = GetChildItem( hNode );
  while ( index >= 0 && child ) {
    index--;
    result = RecourseEnumeration( child, index );
    child = GetNextSiblingItem( child );
  }  
  return result;
}
    

//      
// ---
bool CTreeWnd::RecourseEnumeration( HTREEITEM hNode, HTREEITEM find, int &index ) {
  if ( hNode != find ) {
		HTREEITEM child = GetChildItem( hNode );
		bool result = false;
		while ( child && !result ) {
			if ( child != find ) {
				index++;
				result = RecourseEnumeration( child, find, index );
				child = GetNextSiblingItem( child );
			}  
			else 
				result = true;
		}
		return result;
	}
	return true;
}

//      
// ---
bool CTreeWnd::RecourseEnumeration( HTREEITEM hNode, int findInd, int &index, int &level ) {
  HTREEITEM child = GetChildItem( hNode );
  bool result = false;
  if ( child ) {
    level++;
    while ( child && !result ) {
      if ( index != findInd ) {
        index++;
        result = RecourseEnumeration( child, findInd, index, level );
        child = GetNextSiblingItem( child );
      }  
      else 
        result = true;
    }
    if ( !result )
      level--;
  }
  return result;
}


//        
// ---
HTREEITEM CTreeWnd::NodeByIndex( int ind ) {
  int index = ind;
  return RecourseEnumeration( /*GetRootItem()*/NULL, index );
}


//        
// ---
void CTreeWnd::ExpandNode( int ind ) {    
  HTREEITEM treeNode = NodeByIndex( ind );
  if ( treeNode )
    Expand( treeNode, TVE_EXPAND );
}  


//        
// ---
void CTreeWnd::CollapseNode( int ind ) {    
  HTREEITEM treeNode = NodeByIndex( ind );
  if ( treeNode )
    Expand( treeNode, TVE_COLLAPSE /*| TVE_COLLAPSERESET*/ );
}  


//    
// ---
void CTreeWnd::SelectNode( int ind ) {    
  HTREEITEM treeNode = NodeByIndex( ind );
//  if ( treeNode )   
    Select( treeNode, TVGN_CARET );
}
    

//        
// ---
int CTreeWnd::GetSelectedIndex() {    
  int index = 0;
  RecourseEnumeration( /*GetRootItem()*/NULL, GetSelectedItem(), index );
  return index;  
}

//    
// ---
bool CTreeWnd::IsExpanded( int ind ) {    
  return IsExpanded( NodeByIndex(ind) );
}


//    
// ---
bool CTreeWnd::IsExpanded( HTREEITEM treeNode ) {    
  if ( treeNode ) {
    if ( GetChildItem(treeNode) ) {
      uint state = TVIS_EXPANDED;
      state = GetItemState( treeNode, state );
      return bool(!!(state & TVIS_EXPANDED));
    }
    return true;
  }
  return false;
}


//    
// ---
bool CTreeWnd::IsNodeVisible( int ind ) {
  bool visible = true;
  for ( --ind; ind >= 0 && visible; ind-- )
    visible = IsExpanded( ind );
  return visible;  
}


//      
// ---
int CTreeWnd::GetNodeLevel( int ind ) {
  int index = 0;
  int level = 0;
  RecourseEnumeration( /*GetRootItem()*/NULL, ind, index, level );
  return level;  
}

//     
// ---
int CTreeWnd::GetNodeLevel( HTREEITEM hNode ) {
  int level = 1;
  HTREEITEM parent = GetParentItem( hNode );
  while ( parent ) {
    parent = GetParentItem( parent );
    level++;
  }  
  return level;  
}


//  
// ---
bool CTreeWnd::HasStyle( uint32 mask ) {
	return !!(GetStyle() & mask);
}


// ---
HTREEITEM CTreeWnd::AddItemChild( HTREEITEM hNode, CTreeItemInfo &newNode ) {
  newNode.hParent      = hNode;
  newNode.hInsertAfter = TVI_LAST;
/*
	if ( IsStateImageType() ) {
		newNode.item.mask |= TVIF_STATE;
		newNode.item.state = INDEXTOSTATEIMAGEMASK(1);
		newNode.item.stateMask = TVIS_STATEIMAGEMASK;
	}
	else {
		newNode.item.mask |= TVIF_IMAGE|TVIF_SELECTEDIMAGE;
		newNode.item.iImage = 0;
		newNode.item.iSelectedImage = 0;
	}
*/
  return InsertItem( &newNode );
}


// ---
HTREEITEM CTreeWnd::InsertItemChild( HTREEITEM hParentNode, HTREEITEM hAfterNode, CTreeItemInfo &newNode ) {
  newNode.hParent      = hParentNode;
  newNode.hInsertAfter = hAfterNode == hParentNode ? TVI_FIRST : hAfterNode;
/*
	if ( IsStateImageType() ) {
		newNode.item.mask |= TVIF_STATE;
		newNode.item.state = INDEXTOSTATEIMAGEMASK(1);
		newNode.item.stateMask = TVIS_STATEIMAGEMASK;
	}
	else {
		newNode.item.mask |= TVIF_IMAGE|TVIF_SELECTEDIMAGE;
		newNode.item.iImage = 0;
		newNode.item.iSelectedImage = 0;
	}
*/
  return InsertItem( &newNode );
}


//    
// ---
HTREEITEM CTreeWnd::AddNode( CTreeItemInfo &newNode, HTREEITEM addAfter ) {
	//  
	lastAddLevel = lastAddLevel ? lastAddLevel : GetNodeLevel( addAfter );
	lastAddNode  = lastAddNode ? lastAddNode : addAfter;

  if ( GetCount() >= 1 ) {
    //      
    if ( newNode.level != 1 && !HasStyle(TVS_LINESATROOT) ) {
      //      -   "  "
      ModifyStyle( 0, TVS_LINESATROOT );
		  ModifyStyle( 0, TVS_HASLINES | TVS_HASBUTTONS );
			SetRedraw( TRUE );
		}

    if ( lastAddLevel == newNode.level ) {
      //    ,    - 
      //       
      lastAddNode = AddItemChild( GetParentItem(lastAddNode), newNode );
      lastAddLevel = newNode.level;
    }  
    else { 
      if ( lastAddLevel <= newNode.level ) {
        //    -   
        //     
        lastAddNode = AddItemChild( lastAddNode, newNode );
        lastAddLevel = newNode.level;
      }  
      else {
        //  -      
        HTREEITEM parent = GetParentItem(lastAddNode);
        while ( parent && GetNodeLevel(parent) > newNode.level ) 
          parent = GetParentItem(parent);
        if ( parent ) {  
          lastAddNode = AddItemChild( GetParentItem(parent), newNode );
          lastAddLevel = newNode.level;
        }
      }   
    }  
  }
  else {
    if ( HasStyle(TVS_LINESATROOT) ) {
      //        -   "  "
      ModifyStyle( TVS_LINESATROOT, 0 );
			SetRedraw( TRUE );
		}

    //       
    lastAddNode  = AddItemChild( /*GetRootItem()*/NULL, newNode );
    lastAddLevel = newNode.level;
  }

	return lastAddNode;
}


//    
// ---
HTREEITEM CTreeWnd::InsertNode( CTreeItemInfo &newNode, HTREEITEM addAfter ) {
	//  
	lastAddLevel = lastAddLevel ? lastAddLevel : GetNodeLevel( addAfter );
	lastAddNode  = lastAddNode ? lastAddNode : addAfter;

  if ( GetCount() >= 1 ) {
    //      
    if ( newNode.level != 1 && !HasStyle(TVS_LINESATROOT) ) {
      //      -   "  "
      ModifyStyle( 0, TVS_LINESATROOT );
		  ModifyStyle( 0, TVS_HASLINES | TVS_HASBUTTONS );
			SetRedraw( TRUE );
		}

    if ( lastAddLevel == newNode.level ) {
      //    ,    - 
      //       
      lastAddNode = InsertItemChild( GetParentItem(lastAddNode), lastAddNode, newNode );
      lastAddLevel = newNode.level;
    }  
    else { 
      if ( lastAddLevel <= newNode.level ) {
        //    -   
        //     
        lastAddNode = AddItemChild( lastAddNode, newNode );
        lastAddLevel = newNode.level;
      }  
      else {
        //  -      
        HTREEITEM parent = GetParentItem(lastAddNode);
        while ( parent && GetNodeLevel(parent) > newNode.level ) 
          parent = GetParentItem(parent);
        if ( parent ) {  
          lastAddNode = AddItemChild( GetParentItem(parent), newNode );
          lastAddLevel = newNode.level;
        }
      }   
    }  
  }
  else {
    if ( HasStyle(TVS_LINESATROOT) ) {
      //        -   "  "
      ModifyStyle( TVS_LINESATROOT, 0 );
			SetRedraw( TRUE );
		}

    //       
    lastAddNode  = AddItemChild( /*GetRootItem()*/NULL, newNode );
    lastAddLevel = newNode.level;
  }

	return lastAddNode;
}


//    
// ---
void CTreeWnd::DeleteSubTree( HTREEITEM parentNode, bool bRootIncluded ) {
  HTREEITEM child = GetChildItem( parentNode );
  while ( child ) {
		HTREEITEM nextChild = GetNextSiblingItem( child );
		DeleteSubTree( child );
		DeleteItem( child );
    child = nextChild;
  }  
	if ( bRootIncluded )
		DeleteItem( parentNode );
}


//     
// ---
void CTreeWnd::SetNodeData( uint32 data, int ind ) {
  HTREEITEM treeNode = NodeByIndex( ind );
  if ( treeNode )
    SetItemData( treeNode, data );
}


//     
// ---
uint32 CTreeWnd::GetNodeData( int ind ) {
  uint32 data = 0;
  HTREEITEM treeNode = NodeByIndex( ind );
  if ( treeNode )
    return GetItemData( treeNode );
  return data;
}

//     (    )
// ---
bool CTreeWnd::IsLeaf( int ind ) {
  return IsLeaf( NodeByIndex(ind) );
}


//     (    )
// ---
bool CTreeWnd::IsLeaf( HTREEITEM treeNode ) {
  if ( treeNode )
    return GetChildItem(treeNode) == NULL;
  return true;  
}


//    
// ---
int CTreeWnd::IndexNode( HTREEITEM hNode ) {
	if ( hNode ) {
		int index = 0;
		RecourseEnumeration( /*GetRootItem()*/NULL, hNode, index );
		return index; 
	}
	return -1;
}


//    "  "
// ---
HTREEITEM CTreeWnd::ForEachEnumeration( HTREEITEM hNode, TvActionFunc action, void *param, int &index ) {
  HTREEITEM result = hNode;
  HTREEITEM child = GetChildItem( hNode );
  while ( child ) {
		index++;
    action( *this, child, index, param );
    result = ForEachEnumeration( child, action, param, index );
    child = GetNextSiblingItem( child );
  }  
  return result;
}
    


//  " "
// ---
void CTreeWnd::ForEach( TvActionFunc action, void *param, HTREEITEM fromNode ) {
  int index = fromNode ? IndexNode( fromNode ) : -1;
  ForEachEnumeration( /*GetRootItem()*/fromNode, action, param, index );
}


//    "   "
// ---
HTREEITEM CTreeWnd::FirstThatEnumeration( HTREEITEM hNode, TvCondFunc action, void *param, int &index, bool &result ) {
  HTREEITEM child = GetChildItem( hNode );
  HTREEITEM resNode = child;
  while ( child && !result ) {
    index++;
    if ( !action(*this, child, index, param) ) {
      resNode = FirstThatEnumeration( child, action, param, index, result );
      child = GetNextSiblingItem( child );
    }  
    else {
      result = true;
      resNode = child;
      break;
    }  
  }
  return resNode;
}


//  "  "
// ---
HTREEITEM CTreeWnd::FirstThat( TvCondFunc action, void *param, HTREEITEM fromNode ) {
  int index = fromNode ? IndexNode( fromNode ) : -1;
  bool result = false;
	HTREEITEM node = FirstThatEnumeration( /*GetRootItem()*/fromNode, action, param, index, result );
  return node && result ? node : NULL;
}


//     
// ---
HTREEITEM CTreeWnd::ItemFromMousePos() {
  CPoint point;
  ::GetCursorPos( &point );
  ::MapWindowPoints( NULL, GetSafeHwnd(), &point, 1 );
  return HitTest( point );
}


BEGIN_MESSAGE_MAP(CTreeWnd, CTreeCtrl)
	//{{AFX_MSG_MAP(CTreeWnd)
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_RBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTreeWnd message handlers

// ---
void CTreeWnd::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {
	if ( nChar == VK_MULTIPLY/*'*'*/ ) {
		SetPossibleRedraw( false );
		SetRedraw(FALSE);
	}

  CTreeCtrl::OnKeyDown( nChar, nRepCnt, nFlags );

	if ( nChar == VK_MULTIPLY/*'*'*/ ) {
		SetPossibleRedraw( true );
		SetRedraw(TRUE);
	}
}

// ---
void CTreeWnd::OnChar( UINT nChar, UINT nRepCnt, UINT nFlags ) {
  CTreeCtrl::OnChar( nChar, nRepCnt, nFlags );
}


// ---
void CTreeWnd::OnRButtonDown( UINT nFlags, CPoint point ) {
  HTREEITEM selection = HitTest( point );

  if ( selection ) 
    SelectItem( selection );
	
	CTreeCtrl::OnRButtonDown(nFlags, point);
}

/////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

// ---
HFONT CCTFontCache::Find( DWORD dwStyle ) {
	for ( int i=0,c=Count(); i<c; i++ ) {
		if ( (*this)[i].m_dwStyle == dwStyle )
			return (*this)[i].m_hFont;
	}
	return NULL;
}

// ---
void CCTFontCache::Flush() {
	for ( int i=0,c=Count(); i<c; i++ ) {
		::DeleteObject( (*this)[i].m_hFont );
	}
}

// ---
CCheckTreeItem::CCheckTreeItem() :
  fChecked( ie_Unchecked ),
	fEnabled( true ),						      // 
  fNode   ( false ),
	fMarkable( true ),
	fExpanded( false ),
	fSelected( false ),
	fFirstVisible( false ),
	fOwnsData( false ),
	fStrictlyDisabled( false ),
  m_Text( "" ),
	m_TreeItem( NULL ),
	m_Font( NULL ),
	m_Data( NULL ),
  m_NumItems( 0 ),
  m_NumItemsChecked( 0 ),
	m_NumItemsSomeChecked( 0 ),
  m_Level( 1 ),
	m_ImageIndex( ii_LeafUnchecked ),
	m_CurrImageIndex( -1 ) {
}

// ---
CCheckTreeItem::~CCheckTreeItem() {
	if ( IsOwnsData() )
		delete m_Data;
}

// ---
bool CCheckTreeItem::operator == ( const CCheckTreeItem &rItem ) {
	return fChecked								== rItem.fChecked								&&
//				fEnabled									== rcItem.fEnabled								&&  
				 fNode									== rItem.fNode									&&
				 fMarkable							== rItem.fMarkable							&&
//				 fExpanded							== rItem.fExpanded							&&  
				 m_Text									== rItem.m_Text									&&
//				 m_TreeItem							== rItem.m_TreeItem							&&   !  
				 m_NumItems							== rItem.m_NumItems							&&
				 m_NumItemsChecked			== rItem.m_NumItemsChecked			&&
				 m_NumItemsSomeChecked	== rItem.m_NumItemsSomeChecked	&&
				 m_Level								== rItem.m_Level								&&
//				 m_CurrImageIndex				== rItem.m_CurrImageIndex     &&
				 m_ImageIndex						== rItem.m_ImageIndex;
}
      

// --- 
void CCheckTreeItem::Assign( const CCheckTreeItem &rItem ) {
	fChecked							= rItem.fChecked;
//	fEnabled									= rcItem.fEnabled								;
  fNode									= rItem.fNode;
  fMarkable							= rItem.fMarkable;
  fExpanded							= rItem.fExpanded;
  fSelected							= rItem.fSelected;
  fFirstVisible					= rItem.fFirstVisible;
  m_Text								= rItem.m_Text;
  m_TreeItem						= NULL;
  m_NumItems						= rItem.m_NumItems;
  m_NumItemsChecked			= rItem.m_NumItemsChecked;
  m_NumItemsSomeChecked	= rItem.m_NumItemsSomeChecked;
  m_Level								= rItem.m_Level;
  m_CurrImageIndex			= -1;
  m_ImageIndex					= rItem.m_ImageIndex;
}


// ---
CCheckTreeItem *CCheckTreeItem::Duplicate() {
	return (new CCheckTreeItem)->operator = ( *this );
}


// --- 
void CCheckTreeItem::SetText( const CString &setText ) {
  m_Text = setText;
}

// --- 
CString &CCheckTreeItem::GetText() {
  return m_Text;
}


// ---
int CCheckTreeItem::GetImageIndex() {
	return int( IsMarkable()
							? IsNode()
								? IsChecked()
									? GetChecked() == ie_SomeChecked
										? ii_NodeSomeChecked
										:	ii_NodeChecked
									: ii_NodeUnchecked
								:	IsChecked()
									? ii_LeafChecked 
									: ii_LeafUnchecked
							: 0
						); 
}


//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
// ---
CCTItemsArray::CCTItemsArray( bool owns ) : 
  CPSyncArray<CCheckTreeItem>( 100, 1, owns ) {
}

// ---
CCTItemsArray::~CCTItemsArray() {
	RemoveAll();
}


// ---
void CCTItemsArray::RemoveAt( int nIndex ) { 
	CPSyncArray<CCheckTreeItem>::RemoveInd( nIndex );
}


// ---
int CCTItemsArray::FindIt( CCheckTreeItem *item ) {
	for ( int i=0, c=GetSize(); i<c; i++ )
		if ( (*this)[i] == item )
			return i;
	return -1;
}

// ---
void CCTItemsArray::InsertAt( int nIndex, CCTItemsArray *pNewArray ) {
	for ( int i=0,c=pNewArray->GetSize(); i<c; i++ )
		Insert( nIndex++, (*pNewArray)[i] );
}

// ---
void CCTItemsArray::InsertAt( int nIndex, CCheckTreeItem *pNewItem ) {
	Insert( nIndex, pNewItem );
}


// ---
bool CCTItemsArray::operator == ( const CCTItemsArray &rItems ) {
	if ( GetSize() == rItems.GetSize() ) {
		for ( int i=0, c=GetSize(); i<c; i++ )
			if ( *(*this)[i] != *rItems[i] )
				return false;
		return true;
	}
	return false;
}

// ---
void CCTItemsArray::operator = ( const CCTItemsArray &rItems ) {
	RemoveAll();

	SetOwns( rItems.GetOwns() );

	for ( int i=0, c=rItems.GetSize(); i<c; i++ ) 
		Add( rItems[i]->Duplicate() );
}


//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

static UINT mAfterLoadExpanding = ::RegisterWindowMessage( _T("AfterLoadExpanding") ); 

// CCheckTree

// ---
CCheckTree::CCheckTree() :
	m_hOldFont( NULL ),
	m_MsgReceiver( NULL ),
	m_hAccelTable( NULL ),
	m_bAccelTableShouldBeDestroyed( false ),
	m_bInterfaceLocked( false ),
	m_nInterfaceLockCount( 0 ),
  m_Items( NULL ), 
  m_ImageList(),
	m_nFirstVisibleInd( -1 ),
	fSpreadMark( true ),				//     
	fStateImageType( false ),		// Image  State
	fOwnsItemsArray( true ),		//  ( )  
	fSetupImagesAtLoad( true ),	//    
	fEnableOpOnDisabled( true ), //     
	fEnableChangeItemsState( true ), //    
	fUseCheckSemantic( true ),				//   CheckTree
	fEnablePostLoadProcessing( true ) //    
{
}


// ---
CCheckTree::~CCheckTree() {
	if ( IsOwnsItemsArray() )
		delete m_Items;

	if ( m_bAccelTableShouldBeDestroyed )
		::DestroyAcceleratorTable( m_hAccelTable );
}


// ---
void CCheckTree::StartupInitiation() {
	::SendMessage( *this, WM_SETFONT, ::SendMessage(::GetParent(*this), WM_GETFONT, 0, 0), 0 );

	CreateImageList();

	ModifyStyle( 0, TVS_HASBUTTONS | TVS_HASLINES );	
	
	LoadTreeWindow();
}


// ---
void CCheckTree::CreateImageList() {
	m_ImageList.Create( IsStateImageType() 
											? IDB_WAS_CHECKLISTSTATEIMAGE 
											: IDB_WAS_CHECKLISTIMAGE,
											20, 1, RGB(255, 255, 255) );
	SetImageList( &m_ImageList, IsStateImageType() ? TVSIL_STATE : TVSIL_NORMAL );
}



// ---
void CCheckTree::PreSubclassWindow() {
	CTreeWnd::PreSubclassWindow();

	if ( ::IsDlgClassName(*GetParent()) ) 
		StartupInitiation();
}


// ---
BOOL CCheckTree::PreTranslateMessage( MSG* pMsg ) {
	if ( m_hAccelTable && (pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN) )
		if ( ::GetFocus() == HWND(*this) && ::TranslateAccelerator(*this, m_hAccelTable, pMsg) )
			return TRUE;

	return CTreeWnd::PreTranslateMessage(pMsg);
}

// ---
void CCheckTree::SetAccelTable( HACCEL	hAccelTable, bool bShouldBeDestroyed ) {
	m_hAccelTable = hAccelTable;
	m_bAccelTableShouldBeDestroyed = bShouldBeDestroyed;
}

// ---
void CCheckTree::LockInterface( bool bLock ) {
	if ( bLock ) {
		m_bInterfaceLocked = true;
		m_nInterfaceLockCount++;
	}
	else {
		if ( m_nInterfaceLockCount )
			m_nInterfaceLockCount--;
		if ( !m_nInterfaceLockCount )
			m_bInterfaceLocked = false;
	}
}

// ---
CWnd *CCheckTree::GetMessageReceiver() {
	if ( m_MsgReceiver )
		return m_MsgReceiver;

	return GetParent();
}

// ---
COLORREF CCheckTree::SetBkColor( COLORREF crColor ) {
	if ( m_ImageList.GetSafeHandle() )
		m_ImageList.SetBkColor( crColor );

	return CTreeWnd::SetBkColor( crColor );
}


// ---
void CCheckTree::SetNodeImage( CCheckTreeItem *item, HTREEITEM hNode ) {

  int imageIndex = item->GetImageIndex();
	if ( item->GetCurrImageIndex() != imageIndex ) {
		if ( !hNode )
			hNode = m_Items ? NodeByIndex( m_Items->FindIt(item) ) : NULL;
#if 0
		HTREEITEM hCurrNode = NodeByIndex( m_Items->FindIt(item) );
		if ( hCurrNode != hNode )
			hNode = hCurrNode;
#endif

		if ( IsStateImageType() )
			// "0"-image must be blank
			SetItemState( hNode, INDEXTOSTATEIMAGEMASK(imageIndex + 1), TVIS_STATEIMAGEMASK );
		else
			SetItemImage( hNode, imageIndex, imageIndex );

		item->SetCurrImageIndex( imageIndex );
		if ( GetCount() == 1 )
			SetRedraw( TRUE );
	}
}


//     
// ---
static void SetupNodeChildren( CCheckTree *owner, CCTItemsArray &aItemsArray, int ind ) {
	if ( ind >= 0 ) {
		CCheckTreeItem *item = aItemsArray[ind];

		int firstChildLevel = item->GetLevel() + 1; //   
		int numItems    = 0;
		int numMarkable = 0;

		for ( int i=ind+1,c=aItemsArray.GetSize(); i<c; i++ ) {
			CCheckTreeItem *cItem = aItemsArray[i];
			if ( cItem->GetLevel() == firstChildLevel ) {
				numMarkable += !owner->IsSpreadMark() || cItem->IsMarkable();
				numItems++;
			}
			else  
				if ( cItem->GetLevel() < firstChildLevel ) 
					break;
		}

		item->SetNode( !!numMarkable );  //  
		item->SetNumItems( numItems );
	}
}


//      
// ---
static void SetupNodes( CTreeWnd &owner, HTREEITEM hNode, int ind, void *par ) {
  if ( hNode ) {
    CCTItemsArray &aItemsArray = *(CCTItemsArray *)par;
    CCheckTree    &list  = (CCheckTree &)owner; 
		CCheckTreeItem *item = aItemsArray[ind];
    if ( owner.GetChildItem(hNode) ) {
      //     

      if ( list.IsSpreadMark() ) {
				//      Item
				list.InternalSetItemChecked( item, CCheckTreeItem::ie_Unchecked );//   

				item->SetNumItemsChecked( 0 );  //   
			}

      //     
			::SetupNodeChildren( &list, aItemsArray, ind );
    }  
  }
}


//      
// ---
void CCheckTree::SetupNodes( HTREEITEM fromNode ) {
	CTreeWnd::ForEach( ::SetupNodes, (void *)m_Items, fromNode );
}

//     
// ---
static void SetupImages( CTreeWnd &owner, HTREEITEM hNode, int ind, void *par ) {
  if ( hNode ) {
    CCTItemsArray &m_Items = *(CCTItemsArray *)par;
    CCheckTree    &list  = (CCheckTree &)owner; 
		CCheckTreeItem *item = m_Items[ind];

    if ( !item->IsNode() ) {
      //     -   
			list.SetNodeImage( item, hNode );

      //     
      list.CheckParents( hNode );
    }
    else {
//			if ( item->IsChecked() )
				//    -    
				list.CheckChildren( hNode, ind, CCheckTreeItem::CieChecked(item->IsChecked()) );

			//       
			list.CheckParents( hNode );
    }
  }
}


//     
// ---
void CCheckTree::SetupImages( HTREEITEM fromNode ) {
	CTreeWnd::ForEach( ::SetupImages, (void *)m_Items, fromNode );
}


//   (    )
// ---
HTREEITEM CCheckTree::AddItem( CCheckTreeItem *item, HTREEITEM addAfter ) {
	return AddNode( CTreeItemInfo(item->GetText(), item->GetLevel()), addAfter );
}


// ---
HTREEITEM CCheckTree::InsertItem( CCheckTreeItem *item, HTREEITEM addAfter ) {
	return InsertNode( CTreeItemInfo(item->GetText(), item->GetLevel()), addAfter );
}


// ---
static void SetupInternalInfo( CTreeWnd &owner, HTREEITEM hNode, int ind, void *par ) {
  CCheckTree    &list  = (CCheckTree &)owner; 

	CCheckTreeItem *item = (*(CCTItemsArray *)par)[ind];
	
	item->SetTreeItem( hNode );

	list.SetItemData( hNode, (DWORD)item ); 

	// Item  -   
	item->SetCurrImageIndex( -1 );

	list.SetNodeImage( item, hNode );
}


//  
// ---
void CCheckTree::LoadTreeWindow( HTREEITEM loadAfter, bool bRootInserted ) {
	if ( m_Items && m_Items->GetSize() ) {
		//   - .    .
//		SetRedraw(FALSE);
		
		HWND hwndFocused;
		int selectedInd;
		int visibleInd;

		if ( IsPossibleRedraw() ) {
			hwndFocused = ::GetFocus();

			selectedInd = IndexNode( GetSelectedItem() );
			visibleInd = IndexNode( GetFirstVisibleItem() );
		}

		int parentIndex = IndexNode( bRootInserted ? GetParentItem(loadAfter) : loadAfter );
		parentIndex = parentIndex >= 0 ? parentIndex : 0;

		int firstIndex = bRootInserted ? IndexNode(loadAfter) : parentIndex + !!loadAfter;

		HTREEITEM addAfter = bRootInserted ? GetPrevSiblingItem(loadAfter) : loadAfter;
		if ( !addAfter && bRootInserted )
			addAfter = GetParentItem(loadAfter);

		if ( loadAfter )
			DeleteSubTree( loadAfter, bRootInserted );
		else
			DeleteAllItems();

		NewLoadSession();

		if ( bRootInserted && firstIndex < m_Items->GetSize() ) {
			CCheckTreeItem *item = (*m_Items)[firstIndex];
			
			HTREEITEM hAddedItem = InsertItem( item, addAfter );
			
			item->SetTreeItem( hAddedItem );

			SetItemData( hAddedItem, (DWORD)item ); 
	
			// Item  -   
			item->SetCurrImageIndex( -1 );

			addAfter = loadAfter = hAddedItem;

			parentIndex = IndexNode( hAddedItem );
			parentIndex = parentIndex >= 0 ? parentIndex : 0;

			firstIndex = parentIndex + 1;
		}

		if ( firstIndex < m_Items->GetSize() ) {
			CCheckTreeItem *prItem = loadAfter ? (*m_Items)[parentIndex] : NULL;
			CCheckTreeItem::CieChecked fHowToChecked = prItem 
																								 ? prItem->GetChecked()
																								 : CCheckTreeItem::ie_Unchecked;
		
			int lastLevel  = (*m_Items)[parentIndex]->GetLevel() + !!loadAfter;

			for ( int i=firstIndex,c=m_Items->GetSize(); i<c && (*m_Items)[i]->GetLevel() >= lastLevel; i++ ) {
				//      
				CCheckTreeItem *item = (*m_Items)[i];
				
				HTREEITEM hAddedItem = AddItem( item, addAfter );
				
				::SetupInternalInfo( *this, hAddedItem, i, m_Items );

				if ( item->IsSelected() )
					selectedInd = i;

				if ( item->IsFirstVisible() )
					visibleInd = i;
/*
				item->SetTreeItem( hAddedItem );

				SetItemData( hAddedItem, (DWORD)item ); 
		
				// Item  -   
				item->SetCurrImageIndex( -1 );
*/				
				addAfter = NULL;
			}

			if ( IsUseCheckSemantic() ) {
				//    
				::SetupNodeChildren( this, *m_Items, parentIndex );

				if ( prItem )
					SetItemChecked( parentIndex, fHowToChecked, true );

				//      
				SetupNodes( loadAfter );

				if ( prItem )
					SetItemChecked( parentIndex, fHowToChecked, true );
			}
			//     
			if ( IsSetupImagesAtLoad() ) 
				SetupImages( loadAfter );
		}

		PostLoadProcessing();

		if ( IsPossibleRedraw() ) {
			HTREEITEM hNode = CTreeWnd::NodeByIndex( visibleInd );
			if ( hNode ) {
				EnsureVisible( hNode );

				m_nFirstVisibleInd = visibleInd;

				SendMessage( WM_HSCROLL, MAKEWPARAM(SB_THUMBPOSITION, 0), 0l );
				SendMessage( WM_HSCROLL, MAKEWPARAM(SB_ENDSCROLL, 0), 0l );
			}

			hNode = CTreeWnd::NodeByIndex( selectedInd );
			if ( hNode ) 
				SelectItem( hNode );

			::SetFocus( hwndFocused );
		}

//		SetRedraw( IsPossibleRedraw() );
	}
}
  

// ---
void CCheckTree::LoadTreeWindow( int fromIndex, int toIndex, HTREEITEM hParent, HTREEITEM addAfter ) {
	if ( m_Items && m_Items->GetSize() ) {
//		SetRedraw(FALSE);
		
		HWND hwndFocused;
		int selectedInd;
		int visibleInd;

		if ( IsPossibleRedraw() ) {
			hwndFocused = ::GetFocus();

			selectedInd = IndexNode( GetSelectedItem() );
			visibleInd = IndexNode( GetFirstVisibleItem() );
		}

		int parentIndex = IndexNode( hParent );
		parentIndex = parentIndex >= 0 ? parentIndex : 0;

		NewLoadSession();

		CCheckTreeItem *prItem = addAfter ? (*m_Items)[parentIndex] : NULL;
		CCheckTreeItem::CieChecked fHowToChecked = prItem 
																							 ? prItem->GetChecked()
																							 : CCheckTreeItem::ie_Unchecked;
	
		CSArray<HTREEITEM> rcParentStack;
		CSArray<HTREEITEM> rcAfterStack;
		int nPIndex = parentIndex;
		HTREEITEM hAddedItem = hParent;

		for ( int i=fromIndex; i<=toIndex; i++ ) {
			//      
			CCheckTreeItem *item = (*m_Items)[i];
			CCheckTreeItem *parentItem = (*m_Items)[nPIndex];
			
			if ( item->GetLevel() > parentItem->GetLevel() || !hAddedItem ) {
				rcParentStack.Add( hAddedItem );
				rcAfterStack.Add( addAfter );
//				nPIndex = IndexNode( rcParentStack[rcParentStack.MaxIndex()] );
//				nPIndex = nPIndex >= 0 ? nPIndex : 0;
			}
			else {
				if ( item->GetLevel() < parentItem->GetLevel() ) {
					int nDelta = parentItem->GetLevel() - item->GetLevel();
					for ( ; nDelta>0; nDelta-- ) {
						rcParentStack.RemoveInd( rcParentStack.MaxIndex() );
						rcAfterStack.RemoveInd( rcAfterStack.MaxIndex() );
					}
//					nPIndex = IndexNode( rcParentStack[rcParentStack.MaxIndex()] );
//					nPIndex = nPIndex >= 0 ? nPIndex : 0;
				}
			}

			if ( rcParentStack.Count() && rcAfterStack.Count() ) {
				hAddedItem = InsertItemChild( rcParentStack[rcParentStack.MaxIndex()], 
																			rcAfterStack[rcAfterStack.MaxIndex()], 
																			CTreeItemInfo(item->GetText(), item->GetLevel()) );

				rcAfterStack[rcAfterStack.MaxIndex()] = hAddedItem;
			}
			else {
				hAddedItem = InsertItemChild( hAddedItem, addAfter, CTreeItemInfo(item->GetText(), item->GetLevel()) );
			}

			::SetupInternalInfo( *this, hAddedItem, i, m_Items );

			nPIndex = i;
			
			if ( item->IsSelected() )
				selectedInd = i;

			if ( item->IsFirstVisible() )
				visibleInd = i;
		}

		if ( IsUseCheckSemantic() ) {
			//    
			::SetupNodeChildren( this, *m_Items, parentIndex );

			if ( prItem )
				SetItemChecked( parentIndex, fHowToChecked, true );

			//      
			SetupNodes( hParent );

			if ( prItem )
				SetItemChecked( parentIndex, fHowToChecked, true );
		}
		//     
		if ( IsSetupImagesAtLoad() )
			SetupImages( hParent );

		PostLoadProcessing();

		if ( IsPossibleRedraw() ) {
			HTREEITEM hNode = CTreeWnd::NodeByIndex( visibleInd );
			if ( hNode ) {
				EnsureVisible( hNode );

				m_nFirstVisibleInd = visibleInd;

				SendMessage( WM_HSCROLL, MAKEWPARAM(SB_THUMBPOSITION, 0), 0l );
				SendMessage( WM_HSCROLL, MAKEWPARAM(SB_ENDSCROLL, 0), 0l );
			}

			hNode = CTreeWnd::NodeByIndex( selectedInd );
			if ( hNode ) 
				SelectItem( hNode );

			::SetFocus( hwndFocused );
		}

//	  SetRedraw( IsPossibleRedraw() );
	}
}
  

//   
// ---
void CCheckTree::SetupInternalInfo() {
	CTreeWnd::ForEach( ::SetupInternalInfo, (void *)m_Items, NULL );
}


//    
// ---
int CCheckTree::IndexNode( HTREEITEM hNode ) {
	int foundInd = -1;
	if ( m_Items && m_Items->GetSize() ) {
		if ( hNode ) {
			CCheckTreeItem *clItem = (CCheckTreeItem *)GetItemData( hNode );
			ASSERT( clItem != NULL );

			foundInd = m_Items->FindIt( clItem );
		}
		else
			return foundInd;
	}
	return foundInd == -1 && hNode ? CTreeWnd::IndexNode( hNode ) : foundInd;
}


//    
// ---
int CCheckTree::IndexNode( CCheckTreeItem *clItem ) {
	int foundInd = -1;
	if ( m_Items->GetSize() ) {
		foundInd = m_Items->FindIt( clItem );
	}
	return foundInd;
}


//     
// ---
HTREEITEM CCheckTree::NodeByIndex( int ind ) {
	if ( ind >=0 && ind < m_Items->GetSize() ) 
		return (*m_Items)[ind]->GetTreeItem();

	return CTreeWnd::NodeByIndex( ind );
}

// ---
static void ExpandNodes( CTreeWnd &owner, HTREEITEM hNode, int ind, void *par ) {
	CCheckTreeItem *item = (*(CCTItemsArray *)par)[ind];
	if ( item->IsExpanded() != owner.IsExpanded(hNode) )
		owner.Expand( hNode, item->IsExpanded() ? TVE_EXPAND : (TVE_COLLAPSE /*| TVE_COLLAPSERESET*/) );   
}
				

// ---
void CCheckTree::PostLoadProcessing() {
	if ( IsEnablePostLoadProcessing() )
		PostMessage( mAfterLoadExpanding, 0, 0 );
//	ForEach( ::ExpandNodes, (void *)m_Items, -1 );
}

//    
// ---
void CCheckTree::SetItemsArray( CCTItemsArray *itemsArray ) {
	if ( itemsArray ) {

		bool bNeedReload = !m_Items || *m_Items != *itemsArray;

		if ( IsOwnsItemsArray() )
			delete m_Items;

		m_Items = itemsArray;

		if ( bNeedReload )
			LoadTreeWindow();
		else
			SetupInternalInfo();
	}
}


//    ,     
// ---
void CCheckTree::UpdateItemsArray( int fromIndex, CCTItemsArray &itemsArray, bool bRootInserted ) {
	HTREEITEM hNode = NodeByIndex( fromIndex );

	int lastLevel  = fromIndex >= 0 
									 ? (*m_Items)[fromIndex]->GetLevel() + 1
									 : 1;

	if ( bRootInserted )
	  m_Items->RemoveAt( fromIndex );
	else
	  fromIndex++;

	for ( ; fromIndex < m_Items->GetSize() && (*m_Items)[fromIndex]->GetLevel() >= lastLevel; ) 
	  m_Items->RemoveAt( fromIndex );
	
	m_Items->InsertAt( fromIndex, &itemsArray );

	LoadTreeWindow( hNode, bRootInserted );
}


//    ,       
// ---
void CCheckTree::UpdateItemsArrayByRange( int fromIndex, int toIndex, CCTItemsArray &itemsArray ) {
	HTREEITEM hNodeAfter  = NodeByIndex( fromIndex - 1 );
	HTREEITEM hNodeParent = GetParentItem( NodeByIndex(fromIndex) );

	int selectedInd = IndexNode( GetSelectedItem() );

	if ( selectedInd >=	fromIndex && selectedInd <=toIndex )
		SelectItem( NULL );

	for ( int i=toIndex; i>=fromIndex; i-- ) {
		HTREEITEM hNode = NodeByIndex( i );
		if ( hNode )
			DeleteItem( hNode );
	  m_Items->RemoveAt( i );
	}
	
	m_Items->InsertAt( fromIndex, &itemsArray);
	toIndex = fromIndex + itemsArray.GetSize() - 1;

	LoadTreeWindow( fromIndex, toIndex, hNodeParent, hNodeAfter );
}


//     ,     
// ---
void CCheckTree::DeleteItemWithSubtree( int nFromIndex ) {
	SetRedraw( FALSE );

	if ( nFromIndex >= 0 ) {
		HTREEITEM hNode = NodeByIndex( nFromIndex );

		int lastLevel  = (*m_Items)[nFromIndex]->GetLevel() + 1;
		int nDelete = nFromIndex + 1;

		for ( ; nDelete < m_Items->GetSize() && (*m_Items)[nDelete]->GetLevel() >= lastLevel; ) 
			m_Items->RemoveAt( nDelete );
		
		m_Items->RemoveAt( nFromIndex );

		DeleteSubTree( hNode );
		DeleteItem( hNode );

		if ( m_Items->GetSize() <= 1 && HasStyle(TVS_LINESATROOT) ) 
			//          -   "  "
			ModifyStyle( TVS_LINESATROOT, 0 );
	}
	else {
		if ( m_Items )
			m_Items->RemoveAll();
		DeleteAllItems();
		//DeleteSubTree( NULL );
		ModifyStyle( TVS_LINESATROOT, 0 );
	}

	SetRedraw( IsPossibleRedraw() );
}


//    
// ---
void CCheckTree::SetTreeItemText( int nItemIndex, const CString &rcText ) {
	if ( m_Items && nItemIndex < m_Items->GetSize() ) {
		(*m_Items)[nItemIndex]->SetText( rcText );
		SetItemText( NodeByIndex(nItemIndex), rcText );
	}
}


//    
// ---
int CCheckTree::GetParentIndex( int nItemIndex ) {
	if ( nItemIndex >= 0 ) {
		int lastLevel  = (*m_Items)[nItemIndex]->GetLevel();
		for ( nItemIndex--; nItemIndex >= 0 && (*m_Items)[nItemIndex]->GetLevel() >= lastLevel; nItemIndex-- );
		return nItemIndex;
/*
		HTREEITEM hParentNode = GetParentItem( (*m_Items)[nItemIndex]->GetTreeItem() );
		if ( hParentNode )
			return IndexNode( hParentNode );
*/
	}
	return -1;
}

//    
// ---
int CCheckTree::GetFirstVisibleIndex() {
	return m_nFirstVisibleInd = IndexNode( GetFirstVisibleItem() );
}


//     
// ---
int CCheckTree::GetNextSiblingIndex( int nItemIndex ) {
	if ( m_Items && nItemIndex < m_Items->GetSize() ) {
		int lastLevel  = (*m_Items)[nItemIndex]->GetLevel();
		nItemIndex++;
		for ( ; nItemIndex < m_Items->GetSize() && (*m_Items)[nItemIndex]->GetLevel() > lastLevel; nItemIndex++ );
		return (nItemIndex < m_Items->GetSize() && (*m_Items)[nItemIndex]->GetLevel() == lastLevel)
					 ? nItemIndex 
					 : -1;
	}
	return -1;
}

//     
// ---
int CCheckTree::GetFirstChildIndex( int nItemIndex ) {
	if ( m_Items && nItemIndex < m_Items->GetSize() ) {
		if ( nItemIndex >= 0 ) {
			int lastLevel  = (*m_Items)[nItemIndex]->GetLevel() + 1;
			nItemIndex++;
			return ( nItemIndex < m_Items->GetSize() && (*m_Items)[nItemIndex]->GetLevel() == lastLevel )
							? nItemIndex
							: -1;
		}
		return 0;
	}
	return -1;
}


//     ,     
// ---
int CCheckTree::AddItemToParent( CCheckTreeItem *pNewItem, int parentIndex ) {
	//  
//	SetRedraw(FALSE);

	int fromIndex = parentIndex + 1;

	HTREEITEM hParentNode = NodeByIndex( parentIndex );

	if ( parentIndex >= 0 ) {
		//   
		CCheckTreeItem *parentItem = (*m_Items)[parentIndex];

		//   
		pNewItem->SetLevel( parentItem->GetLevel() + 1 );

		if ( !IsLeaf(hParentNode) ) {
			//    
			int lastLevel  = (*m_Items)[fromIndex]->GetLevel();

			for ( ; fromIndex < m_Items->GetSize() && (*m_Items)[fromIndex]->GetLevel() >= lastLevel; fromIndex++ ); 
		}
	}
	else {
		//   
		pNewItem->SetLevel( 1 );
		//    
		fromIndex = m_Items->GetUpperBound() + 1;
	}

	//   
	if ( fromIndex >= 0 )
		m_Items->InsertAt( fromIndex, pNewItem);
	else
		m_Items->Add( pNewItem );

	//   

//  HTREEITEM hAddedItem = AddItemChild( hParentNode, CTreeItemInfo(pNewItem->GetText(), pNewItem->GetLevel()) );

	
  NewLoadSession();

	HTREEITEM hAddAfter = NodeByIndex( fromIndex - 1 );

	HTREEITEM hAddedItem = AddItem( pNewItem, hAddAfter );
			
	//   
	pNewItem->SetTreeItem( hAddedItem );

	//   
	SetItemData( hAddedItem, (DWORD)pNewItem ); 

	//   
	SetNodeImage( pNewItem, hAddedItem );

	//    
  ::SetupNodeChildren( this, *m_Items, parentIndex );

	//      
	SetupNodes( hParentNode );

	//     
	SetupImages( hParentNode );

	//  
//	SetRedraw( m_bPossibleRedraw );

	return IndexNode( hAddedItem );
}


//     ,        
// ---
int CCheckTree::InsertItemToParent( CCheckTreeItem *pNewItem, int parentIndex, int indexBefore ) {
	//  
//	SetRedraw(FALSE);

	int fromIndex = indexBefore;

	HTREEITEM hParentNode = NodeByIndex( parentIndex );

	if ( parentIndex >= 0 ) {
		//   
		CCheckTreeItem *parentItem = (*m_Items)[parentIndex];

		//   
		pNewItem->SetLevel( parentItem->GetLevel() + 1 );
	}
	else {
		//   
		pNewItem->SetLevel( 1 );
	}

	//   
	if ( fromIndex >= 0 )
		m_Items->InsertAt( fromIndex, pNewItem);
	else
		m_Items->Add( pNewItem );

	//   

//  HTREEITEM hAddedItem = AddItemChild( hParentNode, CTreeItemInfo(pNewItem->GetText(), pNewItem->GetLevel()) );

	
  NewLoadSession();

	HTREEITEM hAddAfter = NodeByIndex( fromIndex - 1 );

	HTREEITEM hAddedItem = InsertItemChild( hParentNode, hAddAfter, CTreeItemInfo(pNewItem->GetText(), pNewItem->GetLevel()) );
//	HTREEITEM hAddedItem = AddItem( pNewItem, hAddAfter );
			
	//   
	pNewItem->SetTreeItem( hAddedItem );

	//   
	SetItemData( hAddedItem, (DWORD)pNewItem ); 

	//   
	SetNodeImage( pNewItem, hAddedItem );

	//    
  ::SetupNodeChildren( this, *m_Items, parentIndex );

	//      
	SetupNodes( hParentNode );

	//     
	SetupImages( hParentNode );

	//  
//	SetRedraw( m_bPossibleRedraw );

	return IndexNode( hAddedItem );
}


//   Checked  
// ---
void CCheckTree::SetItemChecked( int iItemIndex, 
																 CCheckTreeItem::CieChecked fHowToChecked, 
																 bool bSpreadAutomatically ) {

	CCheckTreeItem *item = (*m_Items)[iItemIndex];
	HTREEITEM hNode = NodeByIndex( iItemIndex );

  if ( !item->IsNode() ) {
    //    
    //   
    InternalSetItemChecked( item, fHowToChecked );

    //  
		SetNodeImage( item, hNode );
  }
  else {
//		if ( item->IsMarkable() )  
			InternalSetItemChecked( item, fHowToChecked );

		//   
		SetNodeImage( item, hNode );
 
		if ( bSpreadAutomatically )
			//    
			CheckChildren( hNode, iItemIndex, fHowToChecked, false );
	}

	if ( bSpreadAutomatically )
		//    
		CheckParents( hNode );
}



// ---
void CCheckTree::InternalSetItemChecked( CCheckTreeItem *item, 
																 CCheckTreeItem::CieChecked fHowToChecked ) {
  item->SetCheck( fHowToChecked );
}


//    
// ---
void CCheckTree::CheckParents( HTREEITEM hNode ) {
	if ( IsSpreadMark() ) {
		HTREEITEM parentNode = GetParentItem( hNode );
		if ( parentNode ) {
			//     

			int parentIndex = IndexNode( parentNode );
			CCheckTreeItem *prItem = (*m_Items)[parentIndex];

			if ( prItem->IsMarkable() ) {

			//    
			//      
				int numItemsChecked     = 0;
				int numItemsSomeChecked = 0;
				int firstChildLevel  = prItem->GetLevel() + 1;
  
				for ( int i=parentIndex+1,c=m_Items->GetSize(); i<c; i++ ) {
					CCheckTreeItem *item = (*m_Items)[i];
					if ( item->GetLevel() == firstChildLevel ) {
						bool bChecked = item->GetChecked() == CCheckTreeItem::ie_Checked && item->IsMarkable();
						numItemsChecked += bChecked;

						bool bSomeChecked = item->GetChecked() == CCheckTreeItem::ie_SomeChecked && item->IsMarkable();
						numItemsSomeChecked += bSomeChecked;
					}
					else  
						if ( item->GetLevel() < firstChildLevel )
							break;
				}

				//      
				prItem->SetNumItemsChecked( numItemsChecked );
				prItem->SetNumItemsSomeChecked( numItemsSomeChecked );

				CCheckTreeItem::CieChecked fHowToCheck = CCheckTreeItem::CieChecked( ItemCheckingAnalize(prItem) );
				InternalSetItemChecked( prItem, fHowToCheck );
			}

			//  
			SetNodeImage( prItem, parentNode );

			//     
			CheckParents( parentNode );
		}
	}
}


//    
// ---
void CCheckTree::CheckChildren( HTREEITEM hNode, int ind, CCheckTreeItem::CieChecked fHowToChecked, bool bSetCheck ) {
	CCheckTreeItem *prItem = (*m_Items)[ind];

	if ( bSetCheck ) {
		if ( prItem->IsMarkable() && prItem->CanChangeChecked() )
			InternalSetItemChecked( prItem, fHowToChecked );

		//   
		SetNodeImage( prItem, hNode );
	}
 
	if ( IsSpreadMark() ) {
		if ( prItem->GetNumItems() ) {
			//      -      
			int firstChildLevel = prItem->GetLevel() + 1;
			int numItemsChecked = 0;
			int numItemsSomeChecked = 0;
    
			for ( int i=ind+1,c=m_Items->GetSize(); i<c; i++ ) {
				CCheckTreeItem *crItem = (*m_Items)[i];
				if ( crItem->GetLevel() == firstChildLevel ) {
					CheckChildren( NodeByIndex(i), i, fHowToChecked );
					bool bChecked = crItem->GetChecked() == CCheckTreeItem::ie_Checked && crItem->IsMarkable();
					numItemsChecked += bChecked;

					bool bSomeChecked = crItem->GetChecked() == CCheckTreeItem::ie_SomeChecked && crItem->IsMarkable();
					numItemsSomeChecked += bSomeChecked;
				}
				else  
					if ( crItem->GetLevel() < firstChildLevel )
						break;
			}

			//      
			prItem->SetNumItemsChecked( numItemsChecked );
			prItem->SetNumItemsSomeChecked( numItemsSomeChecked );

			CCheckTreeItem::CieChecked fHowToCheck = CCheckTreeItem::CieChecked( ItemCheckingAnalize(prItem) );
			InternalSetItemChecked( prItem, fHowToCheck );

			//   
			SetNodeImage( prItem, hNode );
		}
	}
}


// ---
int CCheckTree::ItemCheckingAnalize( CCheckTreeItem *prItem ) {
	if ( prItem->GetNumItemsChecked() || prItem->GetNumItemsSomeChecked() ) {
/*
		//   
		if ( prItem->GetNumItems() == numItemsChecked ) 
			//      -   
     	return CCheckTreeItem::ie_Checked;
		else 
			//  -   -
     	return CCheckTreeItem::ie_SomeChecked;
*/
   	return CCheckTreeItem::ie_Checked;
	}
	else 
		//    
   	return CCheckTreeItem::ie_Unchecked;
}


//  
// ---
void CCheckTree::CheckNode( HTREEITEM hNode ) {
  int ind = IndexNode( hNode );
	CCheckTreeItem *item = (*m_Items)[ind];
	SetItemChecked( ind, CCheckTreeItem::CieChecked(!item->IsChecked()), true );
}


// 
// ---
void CCheckTree::ForEach( TvActionFunc action, void *param, int indFromNode ) {
	if ( m_Items && indFromNode < m_Items->GetSize() ) {
		int firstChildLevel = indFromNode >= 0 
													? (*m_Items)[indFromNode]->GetLevel() + 1
													: 1; //   

		for ( int i=indFromNode + 1,c=m_Items->GetSize(); i<c; i++ ) {
			CCheckTreeItem *item = (*m_Items)[i];
			if ( item->GetLevel() >= firstChildLevel ) 
				action( *this, item->GetTreeItem(), i, param );
			else  
				break;
		}
	}
/*
	HTREEITEM hItem = indFromNode == -1 ? NULL : NodeByIndex( indFromNode );
	CTreeWnd::ForEach( action, param, hItem );
*/
}


// 
// ---
int CCheckTree::FirstThat( TvCondFunc action, void *param, int indFromNode ) {
	if ( m_Items && indFromNode < m_Items->GetSize() ) {
		int firstChildLevel = indFromNode >= 0 
													? (*m_Items)[indFromNode]->GetLevel() + 1
													: 1; //   

		for ( int i=indFromNode + 1,c=m_Items->GetSize(); i<c; i++ ) {
			CCheckTreeItem *item = (*m_Items)[i];
			if ( item->GetLevel() >= firstChildLevel ) {
				if ( action(*this, item->GetTreeItem(), i, param) ) 
					return i;
			}
			else  
				break;
		}
	}
	return -1;
/*
	HTREEITEM hItem = indFromNode == -1 ? NULL : NodeByIndex( indFromNode );
	return IndexNode( CTreeWnd::FirstThat(action, param, hItem) );
*/
}

// ---
void CCheckTree::SetNodeStyle( CCheckTreeItem *item, DWORD dwStyle ) {
	if ( dwStyle ) {
		HFONT hFont = m_FontCache.Find( dwStyle );
		if ( !hFont ) {
			CFont *pFont = GetFont();
			if ( pFont ) {
				LOGFONT rcLogfont;
				pFont->GetLogFont( &rcLogfont );
				rcLogfont.lfWeight = (dwStyle & CTIS_BOLD) ? FW_BOLD : FW_NORMAL;
				rcLogfont.lfItalic = (dwStyle & CTIS_ITALIC) ? TRUE : FALSE;
				rcLogfont.lfUnderline = (dwStyle & CTIS_UNDERLINE) ? TRUE : FALSE;
				rcLogfont.lfStrikeOut = (dwStyle & CTIS_STRIKEOUT) ? TRUE : FALSE;

				hFont = ::CreateFontIndirect( &rcLogfont );
				m_FontCache.Add( CCTFontCacheItem(hFont, dwStyle) );
			}
		}
		item->SetFont( hFont );
	}
	else 
		item->SetFont( NULL );
}

// ---
void CCheckTree::SetEnablePostLoadProcessing( bool set /*= true*/ ) { 
	fEnablePostLoadProcessing = set;
	PostLoadProcessing();
}


// ---
bool CCheckTree::IsEnablePostLoadProcessing() { 
	return !!fEnablePostLoadProcessing;
} 

#define CTreeCtrl CTreeWnd
BEGIN_MESSAGE_MAP(CCheckTree, CTreeCtrl)
#undef CTreeCtrl
	//{{AFX_MSG_MAP(CCheckTree)
	ON_WM_CREATE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_CHAR()
	ON_WM_NCLBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEACTIVATE()
	ON_NOTIFY_REFLECT(TVN_ITEMEXPANDED, OnItemExpanded)
	ON_NOTIFY_REFLECT(TVN_SELCHANGING, OnSelChanging)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelChanged)
	ON_REGISTERED_MESSAGE(mAfterLoadExpanding, OnAfterLoadExpanding)
	ON_MESSAGE(WM_SETREDRAW, OnSetRedraw)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CCheckTree message handlers

// ---
int CCheckTree::OnCreate( LPCREATESTRUCT lpCreateStruct ) {
	if (CTreeWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	
	if ( !::IsDlgClassName(*GetParent()) )
		StartupInitiation();

	return 0;
}


// ---
void CCheckTree::OnDestroy() {
	if ( m_hOldFont )
		::SelectObject( *GetDC(), m_hOldFont );

	m_FontCache.Flush();

	CTreeWnd::OnDestroy();
}

// ---
static void ExpandNodeUnconditionally( CTreeWnd &owner, HTREEITEM hNode, int ind, void *par ) {
	if ( !owner.IsExpanded(hNode) )
		owner.Expand( hNode, TVE_EXPAND );   
}
				

// ---
int CCheckTree::OnMouseActivate( CWnd* pDesktopWnd, UINT nHitTest, UINT message ) {
	CToolTipCtrl *pTool = GetToolTips();
	if ( pTool ) 
		pTool->ShowWindow( SW_HIDE );

	return CTreeWnd::OnMouseActivate( pDesktopWnd, nHitTest, message );
}


// ---
void CCheckTree::OnNcLButtonDown( UINT nFlags, CPoint point ) {
	CToolTipCtrl *pTool = GetToolTips();
	if ( pTool ) 
		pTool->ShowWindow( SW_HIDE );

	CTreeWnd::OnNcLButtonDown(nFlags, point);
}


// ---
void CCheckTree::OnKeyDown( UINT nChar, UINT nRepCnt, UINT nFlags ) {
	HTREEITEM hNode = GetSelectedItem();
	if ( nChar == VK_SPACE ) {
		if ( IsEnableChangeItemsState() )
			CheckNode( hNode );
	}
	else {
		if ( GetUpdateRect(NULL, TRUE) )
			UpdateWindow();

		bool bExpanded = IsExpanded( hNode );
		bool bControl  = !!getKState(VK_CONTROL);
		bool bRight		 = !!getKState(VK_RIGHT);
		bool bLeft 		 = !!getKState(VK_LEFT);
		if ( bControl && (bRight || bLeft) ) {
			BYTE pKeyArray[256];
			::GetKeyboardState( pKeyArray );
			pKeyArray[VK_CONTROL]	= 0;
			::SetKeyboardState( pKeyArray );
		}

    CTreeWnd::OnKeyDown( nChar, nRepCnt, nFlags );

		if ( !bExpanded && bRight && bControl ) {
			SetPossibleRedraw( false );
			SetRedraw( FALSE );

			CTreeWnd::ForEach( ::ExpandNodeUnconditionally, (void *)m_Items, hNode );
			EnsureVisible( hNode );

			SetPossibleRedraw( true );
			SetRedraw( TRUE );
		}
	}
}

// ---
void CCheckTree::OnChar( UINT nChar, UINT nRepCnt, UINT nFlags ) {
	if ( nChar != VK_SPACE ) 
		CTreeWnd::OnChar( nChar, nRepCnt, nFlags );
}


// ---
void CCheckTree::OnKeyUp( UINT nChar, UINT nRepCnt, UINT nFlags ) {
	if ( nChar != VK_SPACE )
		CTreeCtrl::OnKeyUp(nChar, nRepCnt, nFlags);
}


// ---
void CCheckTree::OnLButtonDblClk(UINT nFlags, CPoint point) {
	UINT flags;
  HTREEITEM hNode = HitTest( point, &flags );
	UINT fCheckFlag = IsStateImageType() ? TVHT_ONITEMSTATEICON : TVHT_ONITEMICON;

  if ( !hNode || !(flags & fCheckFlag) ) 	
	  CTreeWnd::OnLButtonDblClk(nFlags, point);
}


// ---
void CCheckTree::OnLButtonDown(UINT nFlags, CPoint point) {
	UINT flags;
  HTREEITEM hNode = HitTest( point, &flags );

	if ( hNode ) {
		UINT fCheckFlag = IsStateImageType() ? TVHT_ONITEMSTATEICON : TVHT_ONITEMICON;

		if ( flags & fCheckFlag ) {
			SelectItem( hNode );
			if ( IsEnableChangeItemsState() )
				CheckNode( hNode );
		}  
		else {
			bool bExpanded = IsExpanded( hNode );

			CTreeWnd::OnLButtonDown(nFlags, point);

			if ( !bExpanded && (flags & TVHT_ONITEMBUTTON)	&& getKState(VK_CONTROL) ) {
				SetPossibleRedraw( false );
				SetRedraw( FALSE );

				CTreeWnd::ForEach( ::ExpandNodeUnconditionally, (void *)m_Items, hNode );
				EnsureVisible( hNode );

				SetPossibleRedraw( true );
				SetRedraw( TRUE );
			}
		}
	}
	else
		CTreeWnd::OnLButtonDown(nFlags, point);
}


// ---
void CCheckTree::OnLButtonUp(UINT nFlags, CPoint point) {
	UINT flags;
  HTREEITEM hNode = HitTest( point, &flags );
	UINT fCheckFlag = IsStateImageType() ? TVHT_ONITEMSTATEICON : TVHT_ONITEMICON;

  if ( !hNode || !(flags & fCheckFlag) ) 	
  	CTreeWnd::OnLButtonUp(nFlags, point);
}


// ---
void CCheckTree::OnItemExpanded( NMHDR* pNMHDR, LRESULT* pResult ) {
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	HTREEITEM hNode = pNMTreeView->itemNew.hItem;
	ASSERT( hNode != NULL);

	CCheckTreeItem *item = (CCheckTreeItem *)GetItemData( hNode );
	item->SetExpanded( pNMTreeView->action == TVE_EXPAND/*IsExpanded(hNode)*/ );
	
	SetNodeImage( item, hNode );
	
	*pResult = 0;
}


// ---
LRESULT CCheckTree::OnAfterLoadExpanding( WPARAM wParam, LPARAM lParam ) {
	ForEach( ::ExpandNodes, (void *)m_Items, -1 );

	if ( IsPossibleRedraw() ) {
		int nFirstInd = IndexNode( GetFirstVisibleItem() );
		if ( nFirstInd != m_nFirstVisibleInd ) {
			HTREEITEM hNode = CTreeWnd::NodeByIndex( m_nFirstVisibleInd );
			if ( hNode ) 
				SelectSetFirstVisible( hNode );
		}
	}

	return TRUE;
}

// ---
void CCheckTree::OnSelChanging(NMHDR* pNMHDR, LRESULT* pResult) {
	if ( !m_bInterfaceLocked ) {
		NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
		
		int nItemIndex = IndexNode( pNMTreeView->itemOld.hItem );
		if ( nItemIndex >= 0 && nItemIndex < m_Items->GetSize() )
			(*m_Items)[nItemIndex]->SetSelected( false );

		*pResult = 0;
	}
	else
		*pResult = TRUE;
}

// ---
void CCheckTree::OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult) {
	if ( !m_bInterfaceLocked ) {
		NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

		int nItemIndex = IndexNode( pNMTreeView->itemNew.hItem );
		if ( nItemIndex >= 0 && nItemIndex < m_Items->GetSize() )
			(*m_Items)[nItemIndex]->SetSelected( true );

		*pResult = 0;
	}
	else
		*pResult = TRUE;
}

// ---
BOOL CCheckTree::OnCommand( WPARAM wParam, LPARAM lParam ) {
	if ( !CTreeWnd::OnCommand(wParam, lParam) ) {
		HWND hWndCtrl = (HWND)lParam;
		if ( !lParam ) {
			// menu or accelerator
			GetMessageReceiver()->SendMessage( CTWM_DOCOMMAND, LOWORD(wParam), 0 );
			return TRUE;
		}
		return FALSE;
	}

	return TRUE;
}


// ---
LRESULT CCheckTree::OnSetRedraw( WPARAM wParam, LPARAM lParam ) {
	return CWnd::Default();
}