////////////////////////////////////////////////////////////////////
//
//  BASEWORK.H
//  CBaseWork Main class definitions
//  AVP 3.0 Engine
//  Alexey de Mont de Rique [graf@avp.ru], Kaspersky Labs. 1999
//  Copyright (c) Kaspersky Labs.
//
////////////////////////////////////////////////////////////////////

#ifndef __BASEWORK_H
#define __BASEWORK_H

#include <scanapi/scanobj.h>
#include <bases/format/records.h>
#include "workarea.h"
#include "clink.h"

#if defined (BASE_EMULATOR)
 #pragma pack (8)
#elif defined (__LINUX__)
 #pragma pack (1)
#endif

class CBaseWork;

class CBase
{
protected:
        char*   p_Names;
        UINT    n_Names;
		CBaseWork*  pBW;
		BOOL valid;
		BYTE* FileTypeTable;

public:
        _TCHAR*		BaseName;
        CPtrArray*  LinkArrayTable[8];
        void*		RPtrTable[8];
        int         RNumTable[8];

        CBase();        // standard constructor
        ~CBase();

		BOOL IsValid(){ return valid;}

#if defined (BASE_EMULATOR)
        int LoadBase(_TCHAR* baseName, CBaseWork* bw);
#else
        int LoadBase(_TCHAR* baseName, CGlobalNames* gn, CBaseWork* bw);
#endif /* defined BASE_EMULATOR */

//        GetNameCount(){return n_Names;};
#ifdef _MAKE2BYTES
        void    Write2b();
#endif

        void*  DoRecords(WORD Type,CWorkArea* wa);
};

#define HT_MASK 0x0FFF

class CBaseWork
{

protected:
        BOOL Analyze();
        CPtrArray* BaseArray;
		BOOL valid;

public:
		BOOL IsValid(){ return valid;}
        CPtrArray* ExcludeArray;
        CPtrArray* MaskArray;
	int CheckWildcards(_TCHAR* name, CPtrArray* array);
        BOOL CheckFalseAlarm();
        void FreeHash();
        CPtrArray** HT;
        BOOL  RegisterHash(R_File* rptr);
        BOOL  ProcessRedundant(AVPScanObject* ScanObject);
        BOOL  NextVolume();
        BOOL  CheckMemory(AVPScanObject* ScanObject);
        void* DoRecords(WORD Type);
        int   ProcessArchive(DWORD offset);
        BOOL  ProcessPacked(AVPScanObject* ScanObject);
        int   AddMask(_TCHAR* ch,CPtrArray* array);
        int   CheckMask(_TCHAR* fullname);
		DWORD ErrorFlush();
        _TCHAR SetName[AVP_MAX_PATH];

#if !defined (BASE_EMULATOR)
        DWORD _DoStdJump;
        DWORD _IsProgram;
        DWORD _GetFirstEntry;
        DWORD _GetNextEntry;
        DWORD _CureStdFile;
        DWORD _CureStdSector;
        DWORD _GetArchOffs;
		CHAR* _Psw;
	
        CGlobalNames GlobalNames;
#endif /* !defined BASE_EMULATOR */

        CWorkArea* WorkArea;
//      AVPScanObject* ScanObject;->in WorkArea.


        CBaseWork();    // standard constructor
        ~CBaseWork();

        LONG LoadBase(_TCHAR* name, int format);
        LONG UnloadBase(_TCHAR* name, int format);
        LONG SetFlags(LPARAM lParam);
        LONG ProcessObject(AVPScanObject* ScanObject);
        LONG Dispatch(AVPScanObject* ScanObject, BYTE SubType);
		_TCHAR* CBaseWork::MakeFullName(_TCHAR* dest,const _TCHAR* src);

        int  CureFile(R_File* rp);
        int  CureSector(R_Sector* rp);
        DWORD DoJump(BYTE* Page);

};

#define CACHE_SIZE 257 //must be > 256 !!!

struct BData
{
	BYTE* DosMemory;
	DWORD*  Trace_Lines;
	DWORD*  Hook_Lines;
	WORD*   MCB_Lines;
	DWORD Num_Hook_Lines;
	DWORD Num_Trace_Lines;
	DWORD Num_MCB_Lines;
	WORD DOS_Mem;
	WORD BIOS_Mem;
	WORD Z_MCB;

	_TCHAR TempPath[512];

	DWORD CacheA[CACHE_SIZE];
	DWORD CacheB[CACHE_SIZE];
	DWORD CacheH[CACHE_SIZE];

	CBase* BasePtr;
    LONG CancelProcess;
	BOOL DiskFull;

	__int64 BombSize;
	BYTE BP;
	DWORD BPCount;
	CPtrArray* AllocRegistry;
	HANDLE MainObjectHandle;
	int WarLen;
	DWORD   ArchHeaderSum;
	CPtrArray* S_Array;
//	void*   S_Array;
};

#ifdef USE_STATIC_BDATA
extern BData* bdata;
#define DupSetCustomDWord(x,y) bdata=(BData*)y
#else
#define bdata ((BData*)DupGetCustomDWord(0))
#endif

#endif//__BASEWORK_H
