//  dvpapi.h
//  Urmas Rahu, Data Fellows, 1995-96

//	DVP/Gatekeeper for Windows 95: API for PM/V86 mode applications


#ifndef __DVPAPI_H
#define __DVPAPI_H


#define VIRSTOP_Dev_ID			    	0x33f8      // virtual device driver id


#include "dvpapi.rch"								// resources identifiers


#ifndef __RING3__

#ifdef __cplusplus
extern "C" {
#endif
VOID PM_Api_Handler(VMHANDLE vm_, PCLIENT_STRUCT pRegs);
VOID V86_Api_Handler(VMHANDLE vm_, PCLIENT_STRUCT pRegs);
DWORD OnDeviceIoctl(PIOCTLPARAMS pIOC);				// the W32_DEVICEIOCONTROL handler

int NotifyUser_VirusFound(const char * msg_);		// this function called when virus found
int NotifyUser_Scanning(int isfile_, const char * msg_);	// this function called upon scanning
int NotifyUser_Error(const char * msg_);			// notify user of error
int NotifyUser_Warning(const char * msg_);			// notify user of warning
int NotifyUser_Debug(const char * msg_);			// send debug message

int Settings_IsScanningEnabled(void);				// returns nonzero if scanning is enabled

int IsNec(void);									// returns nonzero if we are running on NEC PC

int NotifyDvp_SettingsChanged(void);				// called when options have been changed by user

const char * Strings_Get(DWORD id_);				// get a string

#ifdef __cplusplus
}
#endif
#else
#include <windows.h>
#endif

/*
;--------------------------------------------------------------------------
;	DVP95_CallbackData
;
;	The callback data structure will be provided by the 16-bit DLL, who
;	passes it to the VxD when registering the ring-3 callback via the API
;	function 0x0101.
;--------------------------------------------------------------------------
*/

typedef struct DVP95_CallbackData_
	{
	DWORD	fStructSize;		// size if this structure for providing down-compatibility
	DWORD	fCallbackAddress;	// seg:off address of the ring-3 callback
	DWORD	fFunctionNumber;	// id of the function performed
	DWORD	fParam0;			// parameter 0: content depends on function
	DWORD	fFlags0;			// flags 0
	DWORD	fResult;			// returned by the ring-3 callback
    char	fData [1024];		// data buffer: content depends on function
    }
	DVP95_CallbackData;


/*
	DVP95_StringUnit
*/
typedef struct DVP95_StringUnit_
	{
	DWORD	fId;				// string ID
	char *	fPtr;				// pointer to string; seg:off in 16-bit DLL, converted to flat in VxD
    }
	DVP95_StringUnit;


/*
	DVPNT BOOTSCANSTRUCT
*/

#define BOOTSECTOR_REPORT_STRING_MAX 128

typedef struct BOOTSCANSTRUCT_
    {
    unsigned char Disk;	            // if reading boot sector 'A','B', ...   if reading MBR '\0, '\1'...
    BOOL Infected;
    BOOL ReadBoot;	        // TRUE = scan boot sector, FALSE = scan mbr
	unsigned char Buffer[BOOTSECTOR_REPORT_STRING_MAX];
    }
    BOOTSCANSTRUCT, *PBOOTSCANSTRUCT;

#                                                    		// function IDs, passed in fFunctionNumber

#define DVPFN_VIRUS 				0x00000001		// 0x0001: virus found
#define DVPFN_SCANNOTIF				0x00000010		// 0x0010: scanning notification
#define DVPFN_ERROR					0x00000020		// 0x0020: error
#define DVPFN_WARNING				0x00000021		// 0x0021: warning
#define DVPFN_DEBUG					0x00000030		// 0x0030: debug message

#define DVPF0_SCANNOTIF_FILE		0x00000000		// flags0: scanned file
#define DVPF0_SCANNOTIF_BOOT		0x00000001		// flags0: scanned boot sector



#															// Win32 DeviceIoControl function IDs

#define DVPIOC_GETVERSION			0x00000777      // get version
#define DVPIOC_ENABLESCAN			0x00000200		// enable/disable scanning
#define DVPIOC_SETOPTION			0x00000210		// set option
#define DVPIOC_SETOPTION2			0x00000211		// set buffer/struct option
#define DVPIOC_SETTINGSCHANGED		0x00000400		// notify dvp.vxd about changed settings
#define DVPIOC_GETSCANCOUNTS		0x00000401		// retrieve counters of scanned objects
#define DVPIOC_SETLANGUAGE			0x00000501		// set language

#define	DVPIOC_ISSERVER				0x00000889	    // is the gatekeeper device a server version
#define	DVPIOC_SCANBOOTSECTOR		0x00000890	    // scan the desired bootsector

#define DVPIOC_REGISTER_THREAD      0x00002068      // register my thread to be able to open files without GK \
#define DVPIOC_UNREGISTER_THREAD    0x00002069      // notifying about it, should unregister as well,
                                                    // thread registering only valid under NT


#															// option IDs, for both IOC and PM API

#define DVPOPTION_SCANFLOPPIES			0x0000		// scan floppies
#define DVPOPTION_SCANHDDS				0x0001		// scan local hard drives
#define DVPOPTION_SCANNETDRIVES			0x0002		// scan net drives
#define DVPOPTION_SCAN					0x0007		// enable scan
#define DVPOPTION_ACTION				0x0010		// action
#define DVPOPTION_PROMPTUSER			0x0018		// prompt user before denying access to infected file

#define DVPOPTION_SCANEXTENSIONS		0x8000		// extensions for executable (scanned) files
#define DVPOPTION_DRIVETYPES			0x8001		// drive type flags (not an option really...)

//////////
//
// The IOCTL structure in NT is defined so that the ioctl code determines whether the
// message has parameters and what type they are. The user-defined message area begins
// at 0x800 for ioctls with parameters. All F-PROT ioctls are mapped to this range.
//
// 23.8.1996 Ismo Bergroth/Data Fellows Ltd.
//
//////////

#define DVPIOC_GETVERSION_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_GETVERSION, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_ENABLESCAN_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ENABLESCAN, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SETOPTION_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETOPTION, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SETOPTION2_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETOPTION2, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SETTINGSCHANGED_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETTINGSCHANGED, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_GETSCANCOUNTS_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_GETSCANCOUNTS, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SETLANGUAGE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETLANGUAGE, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_ENABLEDEBUGLOG_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ENABLEDEBUGLOG, METHOD_BUFFERED,\
		 FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_ISSERVER_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ISSERVER, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SCANBOOTSECTOR_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCANBOOTSECTOR, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_REGISTER_THREAD_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_REGISTER_THREAD, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_UNREGISTER_THREAD_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_UNREGISTER_THREAD, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#ifdef __RING3__									// this stuff only used for ring-3 stuff

#ifdef __cplusplus
extern "C" {
#endif

								// the callback definition
typedef int (FAR PASCAL far * DVP95Callback) (/*DVP95_CallbackData far **/void);


// Get the driver's version; ret 0:OK, -1:driver not installed, -2:error
// version_ will be major:minor:letter:extra (MSB->LSB)
int far _export DVP95GetVersion(unsigned long far * version_);

// Query activation; ret 0:OK, -1:driver not installed, -2:error
int far _export DVP95QueryActivation(int far * active_, int far * type_);

// Set machine type (0==PC AT, 1==NEC PC); 0:OK, -1:driver not installed, -2:error
int far _export DVP95SetComputerType(int type_);

// Enable or disable the callback; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95EnableCallback(int enable_);


// Register/unregister the callback; ret 0:OK, -1:driver not installed, -2:error.
// Pass NULL to unregister the callback.
int far _export DVP95RegisterCallback(DVP95_CallbackData far * callbackdata_);



// Enable/disable scan notifications; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95EnableScanNotifications(int enable_);

// Enable/disable debug notifications; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95EnableDebugNotifications(int enable_);

// Set option value; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95SetOption(WORD optionid_, WORD value_);

// Set buffer/struct option value; ret 0:OK, -1:d1river not installed, -2:error.
int far _export DVP95SetOption2(WORD optionid_, void far * value_);

// Register strings; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95RegisterStrings(DVP95_StringUnit far * table_, int count_);

// Set language; ret 0:OK, -1:driver not installed, -2:error.
int far _export DVP95SetLanguage(int languageindex_);

// Set/remove file system hook; ret 0:OK, -1:driver not installed, -2:error
//int far _export DVP95FileSystemHook(int set_);



// Load F-PROTC.VXD; returns error code, zero on success
int far _export DVP95Load(const char far * fullname_);

// Unload F-PROTC.VXD; returns error code, zero on success
int far _export DVP95Unload(void);


// Register the notification window for receiving messages
void far pascal _export SetNotificationWindow(HWND hwnd_);

#ifdef __cplusplus
}
#endif

#endif


#endif //__DVPAPI_H


