/****************************************************************************\
  FILE..........: fsmadef.h
  COPYRIGHT.....: Copyright (c) 1998, Data Fellows Ltd.
  VERSION.......: 1.00
  DESCRIPTION...: F-Secure Management Agent Plug-in API macros and
                  type declarations
  HISTORY.......: DATE       COMMENT
                  ---------- ------------------------------------------------
                  1998-07-28 Created - Alexander Paperno (AP)
\****************************************************************************/

/*===========================================[ REDEFINITION DEFENCE ]=======*/

#ifndef FSMADEF_H
#define FSMADEF_H

/*===========================================[ SPECIAL ]====================*/
/*===========================================[ IMPORT DECLARATIONS ]========*/
/*===========================================[ PUBLIC MACROS ]==============*/

#define FSMA_DECLARE_HANDLE(name) \
		struct name##__ { int unused; }; typedef struct name##__ *name

#define FSMA_MESSAGE(Priority, Customer, Facility, MessageID) \
		((MSGID)                            \
		((unsigned long)(MessageID)       + \
		((unsigned long)(Facility) << 16) + \
		((unsigned long)(Customer) << 26) + \
		((unsigned long)(Priority) << 28)))

#define FSMA_STATUS(Severity, Customer, Facility, StatusCode) \
		((FSMAERROR)                        \
		((unsigned long)(StatusCode)      + \
		((unsigned long)(Facility) << 16) + \
		((unsigned long)(Customer) << 26) + \
		((unsigned long)(Severity) << 30)))

#define FSMA_ERROR(Customer, Facility, StatusCode) \
		FSMA_STATUS(FSMASEV_ERROR, Customer, Facility, StatusCode)

/*===========================================[ PUBLIC CONSTANTS ]===========*/

/* Customer code bits */
#define FSMA_BIT                          1
#define USER_BIT                          0

/* Facility codes for FSMA components */
#define FSMAF_CORE                        0x00
#define FSMAF_VERSION_CONTROL             0x01
#define FSMAF_INSTALLATION                0x02
#define FSMAF_CONFIGURATION               0x03
#define FSMAF_NETWORK                     0x04
#define FSMAF_USER_INTERFACE              0x05
#define FSMAF_ALERTS                      0x06
#define FSMAF_GATEKEEPER                  0x07
#define FSMAF_LOCALIZATION                0x08

/* severity values */
#define FSMASEV_SUCCESS                   0x0
#define FSMASEV_ERROR                     0x2
#define FSMASEV_INFO                      0x1
#define FSMASEV_WARNING                   0x3

/* priority values */
#define FSMAPR_SYSTEM                     0x00
#define FSMAPR_LOW                        0x01
#define FSMAPR_NORMAL                     0x02
#define FSMAPR_HIGH                       0x03

/* timeouts (in milliseconds) */
#define FSMA_DEF_TIMEOUT                  2000L
#define FSMA_INFINITE                     -1L

/*===========================================[ PUBLIC TYPES ]===============*/

FSMA_DECLARE_HANDLE(FSMASESSION);
FSMA_DECLARE_HANDLE(FSMAMESSAGE);
FSMA_DECLARE_HANDLE(FSMAMESSAGEPUMP);

typedef unsigned long FSMAERROR;
typedef unsigned long MSGID;

/*===========================================[ PSEUDO/INLINE METHODS ]======*/

#define FSMAS_GET_SEVERITY(Status)   (((Status) >> 30) & 0x3)
#define FSMAS_IS_SUCCESS(Status)     (FSMAS_GET_SEVERITY(Status) == FSMASEV_SUCCESS)
#define FSMAS_IS_ERROR(Status)       (FSMAS_GET_SEVERITY(Status) == FSMASEV_ERROR)

/*===========================================[ END REDEFINITION DEFENCE ]===*/

#endif /* FSMADEF_H */

/** (END OF FILE  : fsmadef.h) **********************************************/
