//
//Interface specification between Gatekeeper NT and the rest of world
//Follows mostly the DVP 95 conventions of Urmas Rahu.
//5.6.96 Mikko Kuisma, Data Fellows Ltd.
//
//
//Your Gatekeep.sys should expose these six function services:
//

#define DVPIOC_ENABLESCAN			0x00000200		// enable/disable scanning
#define DVPIOC_SETOPTION			0x00000210		// set option
#define DVPIOC_SETOPTION2			0x00000211		// set buffer/struct option
#define DVPIOC_GETSCANCOUNTS		0x00000401		// retrieve counters of scanned objects
#define DVPIOC_SETLANGUAGE          0x00000501      // set language
#define DVPIOC_ENABLEDEBUGLOG		0x00000511		// keep messages and filenames for reporting

#define	DVPIOC_SCANBOOTSECTOR		0x00000403	// scan the desired bootsector
#define	DVPIOC_SCANFILE				0x00000500	// scan the desired file
#define	DVPIOC_ALLOWTHREAD			0x00000555	// allow thread access to files




/*
//Function calling models
//
//The first parametre (HANDLE) is the handle acquired by calling CreateFile
//with your device name.
//
####
// Get the driver's version; ret 0:OK, -1:driver not installed, -2:error
// version_ will be major:minor:letter:extra (MSB->LSB)
char * version
DWORD bytesreturned;
DeviceIoControl(handle, DVPIOC_GETVERSION, 0, 0, version_, 4, &bytesreturned, 0);


####
// Enable/disable DVP scanning while the on-demand scanner scans.
int enable
DWORD bytesreturned;
DeviceIoControl(handle_, DVPIOC_ENABLESCAN, enable ? "1" : "0", 2, 0, 0, &bytesreturned, 0);


####
//  Sets an option for DVP.VXD.  <optionid_> must be one of the following
//	values  (values under 0x8000 only).
*/

// These three can be neclected - we do all of them. Command introduced them.
#define DVPOPTION_SCANFLOPPIES			0x0000		// scan floppies
#define DVPOPTION_SCANHDDS				0x0001		// scan local hard drives
#define DVPOPTION_SCANNETDRIVES			0x0002		// scan net drives

#define DVPOPTION_SCAN					0x0007		// enable scan
#define DVPOPTION_ACTION				0x0010		// action (0==none,1==rename,2==delete,3==disinfect)
#define DVPOPTION_PROMPTUSER			0x0018		// prompt user before denying access to infected file

/*

DWORD bytesreturned = 0L;
WORD parameters [2];
parameters[0] = optionid_; // One of abobe
parameters[1] = value_;	   // On or off or opttion number
DeviceIoControl(handle_, DVPIOC_SETOPTION, parameters, 4, 0, 0, &bytesreturned, 0);


####
//  Sets an option from a buffer for DVP.VXD.  <optionid_> must be one of the following
//	values  (values over or equal to 0x8000 only).
#define DVPOPTION_SCANEXTENSIONS		0x8000		// extensions for executable (scanned) files

DWORD bytesreturned = 0L;
char buf[MAX_PASS];
*(WORD *)buf = optionid_;
memcpy(buf + sizeof(WORD), value_, valuelen_);
DeviceIoControl(handle_, DVPIOC_SETOPTION2, buf, valuelen_ + sizeof(WORD), 0, 0, &bytesreturned, 0);


####
//  Retrieves the counts of scanned objects.  The counts are returned in buffer
//	pointed to by <counters_> as four unsigned long values:

//		counters_[0]    : number of files scanned in DOS sessions
//		counters_[1]    : number of files scanned in Windows
//		counters_[2]    : number of boot sectors scanned in DOS sessions
//		counters_[3]    : number of boot sectors scanned in Windows

unsigned long counters[];
DWORD bytesreturned = 0L;
DeviceIoControl(handle_, DVPIOC_GETSCANCOUNTS, 0, 0, &counters_, 4 * sizeof(unsigned long), &bytesreturned, 0);


####
//  Called after all the options are given (commit them).

DWORD bytesreturned = 0L;
DeviceIoControl(handle_, DVPIOC_SETTINGSCHANGED, 0, 0, 0, 0, &bytesreturned, 0);

//////////

####
//  Called to enable the logging of scanned files. The messages and filenames are retrieved in batch.
//  Default is off.

DWORD bytesreturned = 0L;
DWORD yes_no;
DeviceIoControl(handle_, DVPIOC_ENABLEDEBUGLOG, 0, 0, yes_no, 0, &bytesreturned, 0);

//////////
// Ismo:
// After you have these services provided in the driver, a possibility to get the
// driver handle by using CreateFile and a Win32 program that communicates (thru one of)
// the driver functions I will start expanding that Win32 program and my exisitng Agent
// by wrapping calls to these functions in appropriate function layers.
//
// I am planning to use a layer program (the Win32 program you start developing) that would
// on initialization start the driver and on quitting stop the driver. This way the DVP stuff
// is isolated from Agent - the same way as we are doing in current Gatekeepers. In addition
// that exe can be made into a service without the overhead that loading the whole agent would
// bring.
//////////

*/

//////////
//
// The IOCTL structure in NT is defined so that the ioctl code determines whether the
// message has parameters and what type they are. The user-defined message area begins
// at 0x800 for ioctls with parameters. All F-PROT ioctls are mapped to this range.
//
// 23.8.1996 Ismo Bergroth/Data Fellows Ltd.
//
//////////



#define DVPIOC_ENABLESCAN_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ENABLESCAN, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA) 

#define DVPIOC_SETOPTION_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETOPTION, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA) 

#define DVPIOC_SETOPTION2_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETOPTION2, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA) 

#define DVPIOC_GETSCANCOUNTS_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_GETSCANCOUNTS, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA) 

#define DVPIOC_SETLANGUAGE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SETLANGUAGE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_ENABLEDEBUGLOG_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ENABLEDEBUGLOG, METHOD_BUFFERED,\
         FILE_READ_DATA | FILE_WRITE_DATA) 



#define DVPIOC_SCANBOOTSECTOR_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCANBOOTSECTOR, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SCANFILE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCANFILE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

// CSS
//
#define DVPIOC_SCAN_MBR_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCAN_MBR, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SCAN_BOOT_SECTOR_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCAN_BOOT_SECTOR, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SCAN_MEMORY_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SCAN_MEMORY, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_GET_SCAN_BUFFER_HANDLE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_GET_SCAN_BUFFER_HANDLE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_RELEASE_SCAN_BUFFER_HANDLE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_RELEASE_SCAN_BUFFER_HANDLE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_SET_SCAN_ACTION_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_SET_SCAN_ACTION, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)



#define DVPIOC_LOAD_SCAN_ENGINE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_LOAD_SCAN_ENGINE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_UNLOAD_SCAN_ENGINE_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_LOAD_SCAN_ENGINE, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_READ_DISK_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_READ_DISK, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_WRITE_DISK_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_WRITE_DISK, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_ALLOWTHREAD_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_ALLOWTHREAD, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)


// DVPIOC_HOLD_REQUEST_NT is no longer used under that name. It has been renamed 
// to DVPIOC_GET_MESSAGE which better describes it's functionality
//
#define	DVPIOC_HOLD_REQUEST			0x00000600	// 'callback'
#define DVPIOC_HOLD_REQUEST_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_HOLD_REQUEST, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_REGISTER_THREAD                  0x00000654
#define DVPIOC_REGISTER_THREAD_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_REGISTER_THREAD, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_UNREGISTER_THREAD                0x00000655
#define DVPIOC_UNREGISTER_THREAD_NT\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+DVPIOC_UNREGISTER_THREAD, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)

#define DVPIOC_GET_TASK_PROGRESS\
	CTL_CODE(FILE_DEVICE_DISK, 0x800+0x657, METHOD_BUFFERED,\
			FILE_READ_DATA | FILE_WRITE_DATA)



